package dbcollection;

import java.util.Vector;

/**
 * Esta clase implementa una factoria abstracta. Los objetos de una tabla
 * relacional son muy facilmente encapsulables en un tipo 
 * <code>Vector</code>. El adaptador 
 * de base de datos adecuado <code>dbObjectAdaptor</code> se utilizara
 * para acceder a la base de datos con el tipo adecuado
 * A su vez, las factorias especificas
 * seran clientes de esta y solo haran de interfaz con CORBA para conservar
 * los tipos. 
 */


public class AbstractFactory 
{
    // Variables internas
    private dbObjectAdaptor dbOA;   // Interfaz objeto-relacinal
    private org.omg.CORBA.ORB orb;  // El ORB
    
    
    static final private int CACHE_MAX = 2048;
    private java.util.Hashtable cache = new java.util.Hashtable(CACHE_MAX);
    
    // Constructor. Se encarga de conectarse a la base de datos
    public AbstractFactory(org.omg.CORBA.ORB orb, dbObjectAdaptor dbOA)
    {
        this.dbOA = dbOA;
        this.orb = orb;
    }

    public java.lang.Object getObject(Vector key)
    {
        if (cache.containsKey(key)) {      
            return cache.get(key);
        } else {
            java.lang.Object tmpObj = dbOA.retrieve(key);
            cacheIt(key,tmpObj);
            return tmpObj;
        }    
    }
    
    private void cacheIt(Vector key, java.lang.Object obj)
    {
        if (cache.size() > CACHE_MAX)
        {
            java.lang.Object tmpKey;
            java.util.Enumeration e = cache.keys();
                
            tmpKey = e.nextElement();
            Vector key2 = new Vector();
            key2.add(tmpKey);
            this.store(key2);
        }

        // Conecta al ORB
        orb.connect((org.omg.CORBA.Object)obj);
        cache.put(key,obj);
    }

    // Este es un interfaz de bajo nivel para acceder directamente a la
    // cache. Los iteradores extraen objetos de la base de datos y los
    // intentan introducir en la cache. 
    public java.lang.Object cacheObject(Vector key, java.lang.Object obj)
    {
        if (cache.containsKey(key)) {      
            return cache.get(key);
        } else {
            cacheIt(key,obj);
            return obj;
        }
    }

    public void remove(Vector key)
    {
        this.store(key);
        dbOA.remove(key);
    }
    
    private void store(Vector key) 
    {
        java.lang.Object tmpObj;
        
        if (cache.containsKey(key)) 
        {
            tmpObj = cache.get(key);
            if (((ChangeSensible)tmpObj).changed()){
                dbOA.store(tmpObj);
            }
            // Desconecta del ORB.
            orb.disconnect((org.omg.CORBA.Object)tmpObj);
        
            // Borra de la cache
            cache.remove(key);
        }
    }
    
        

    public boolean exists(Vector key)
    {
        return cache.containsKey(key) || dbOA.exists(key);
    }

    public void finalize() 
    {
        java.util.Vector tmpKey;
        java.util.Enumeration e = cache.keys();
        
        while (e.hasMoreElements()) 
        {
            tmpKey = (java.util.Vector)e.nextElement();
            this.store(tmpKey);
        }
    }
}

