package dbcollection;

/**
 * El interfaz dbObjectAdaptor identifica a los objetos que pueden hacer
 * de puente entre un objeto CORBA y una base de datos. Para cada clase
 * CORBA destinada a contener un objeto relacional, se crea una clase
 * que implementa este interfaz. Estas clases son necesarias debido
 * a la falta de genericidad en Java. Esto viene obligado debido a que si
 * queremos tener una clase como "AbstractFactory" que pueda tratar con
 * cualquier tipo de objetos, por un lado, tenemos que darle objetos de
 * tipo "Object", pero, por otro, deben ser realmente objetos del tipo
 * CORBA adecuado. Finalmente, se producira una conversion para enviarla
 * al cliente en la clase "ClientFactory".
 */
public interface dbObjectAdaptor extends java.util.Enumeration 
{
    /**  
     * Retorna un objeto (CORBA) del tipo especifico representado por
     * el <code>java.lang.Object</code> retornado. La implementacion
     * especifica de estos metodos dependen de la base de datos que se
     * este utilizando (SQL, ficheros, etc.) y del tipo de objetos a 
     * retornar.
     * @param key Clave primaria del objeto relacional en un vector
     * @return El correspondiente objeto CORBA
     */
    java.lang.Object retrieve(java.util.Vector key);
    
    /** 
     * Almacena un objeto del tipo especifico en la base de datos. 
     * Notese que estas llamadas solo se realizaran para objetos en los
     * que se necesite su guardado, porque hayan sido modificados. Los
     * objetos CORBA correspondientes a las bases de datos implementan
     * el interfaz <code>ChangeSensible</code>, el cual puede ser utilizado
     * para preguntarles si se han modificado, y en ese caso, ser guardados
     * por la <code>AbstractFactory</code> cuando sea necesario.
     * @param obj El objeto CORBA a ser almacenado
     */
    void store(java.lang.Object obj);

    /** 
     * Elimina un objeto de la base de datos.
     * @param key La clave del objeto relacional a ser eliminado
     */
    void remove(java.util.Vector key);

    /** 
     * Comprueba la existencia en la base de datos de un objeto.
     * @param key La clave del objeto relacional a ser comprobado
     * @return true si el objeto existe en la base de datos
     */
    boolean exists(java.util.Vector key);
    
}
