#include <fstream.h>
#include <stdio.h>
#include <stdlib.h>
#include "interfaz.h"
#include <string.h>

void generarSQL(interfaz *inter, ofstream &SQL);
void generarIDL(interfaz *inter, ofstream &IDL);
void generarImpl(interfaz *inter);
void generarFactoryImpl(interfaz *inter);
void generarOA(interfaz *inter);
void generarIteratorImpl(interfaz *inter);
void generarSafe(interfaz *inter);
void generarServidor(list<interfaz*> *linter);
char *nombretabla(char *modulo, char *nombre, char *intermedio);
void imprimeTipoSQL(atributo* atr, ofstream &SQL);
void imprimeTipoIDL(atributo* atr, ofstream &IDL);
void imprimeTipoJava(atributo* atr, ofstream &SAL);
void imprimeObjetoJava(atributo* atr, ofstream &SAL);
void imprimeObjetoJava(tipo_attr tipo, ofstream &SAL);
void imprimeObtenerDatos(atributo* atr, ofstream &SAL,int i, char *resultset);
void imprimeCastingJava(atributo* atr, ofstream &SAL, int nclave);
void imprimeNuloSQL(atributo* atr, ofstream &SAL);
void imprimeNuloJava(atributo* atr, ofstream &SAL);
void imprimeClave(interfaz *inter,ofstream &IMPL);
void imprimeGeneraClave(interfaz *inter,ofstream &IMPL);
void imprimeGeneraSQL(interfaz *inter,ofstream &IMPL);
void imprimeGeneraValoresInsertSQL(interfaz *inter,ofstream &IMPL);
void imprimeGeneraValoresNuevoObjeto(interfaz *inter,ofstream &IMPL);
void tratarherencia(interfaz *inter);


void generarClases(list<interfaz*> *linter, ofstream &SQL, ofstream &IDL, char *nombreIDL) {
   list<interfaz *>::iterator it = linter->begin();
   while ((*it)!=0) {
      SQL << "DROP TABLE " << nombretabla((*it)->modulo,(*it)->nombre,"_") << ";" << endl;
      it++;
   }

   SQL << endl;

   IDL << "#include \"" << nombreIDL << "\"" << endl;

   it = linter->begin();
   while ((*it)!=0) {
      tratarherencia(*it);
      generarSQL((*it),SQL);
      generarIDL((*it),IDL);
      generarImpl(*it);
      generarFactoryImpl(*it);
      generarOA(*it);
      generarIteratorImpl(*it);
      generarSafe(*it);
      it++;
   }
   generarServidor(linter);

}

void tratarherencia(interfaz *inter) {
   if (inter->nombrespadres!=0) {
      list<interfaz *>::iterator it = inter->padres->begin();
   
      while ((*it)!=0) {
         list<atributo *>::iterator it2 = (*it)->atributos->begin();
         while ((*it2)!=0) {
            inter->atributos->push_back(*it2);
            it2++;
         }
         it++;
      }
   }
}

// Sustituye los '.' por intermedio y une el modulo al nombre con otro intermedio
char *nombretabla(char *modulo, char *nombre, char *intermedio) {
   char *ret = new char[strlen(modulo) + strlen(intermedio) + strlen(nombre) + 1];
   int i;
   
   if (modulo[0]!=0) {
      char *aux = new char[strlen(modulo) + 1];
      int principiomod=0;
      strcpy(aux,modulo);
      ret[0]=0;
      for (i=0; i<=strlen(modulo); i++) {
         if (modulo[i]=='.' || modulo[i]==0) {
            aux[i]=0;
            strcat(ret,&aux[principiomod]);
            strcat(ret,intermedio);
            principiomod=i+1;
         }
      }
      strcat(ret,nombre);
      
      delete [] aux;
   }
   else strcpy(ret,nombre);
   
   return ret;
}

void generarSQL(interfaz *inter, ofstream &SQL) {
   cout << "Generando la sentencia SQL para la clase " << inter->nombre << endl;
   SQL << endl;
   SQL << "CREATE TABLE " << nombretabla(inter->modulo,inter->nombre,"_") << " (" << endl;
   
   list<atributo *>::iterator it = inter->atributos->begin();
   while ((*it)!=0) {         
      SQL << "\t" << (*it)->nombre;
      
      // Tipo del atributo
      imprimeTipoSQL((*it),SQL);
      
      SQL << "," << endl;
      it++;
   }
   
   SQL << "\tCONSTRAINT pk_" << nombretabla(inter->modulo,inter->nombre,"_") << " PRIMARY KEY (";
   it = inter->atributos->begin();
   bool primeratr=true;
   while ((*it)!=0) {
      if ((*it)->readonly) {
         // Es un atributo primario
         if (primeratr)
            primeratr=false;
         else
            SQL << ", ";
         
         SQL << (*it)->nombre;
      }
      it++;
   }
   
   SQL << ")" << endl << "\t);" << endl;
   
}

void imprimeTipoSQL(atributo* atr, ofstream &SQL) {
      switch (atr->tipo.tipo) {
         case tipo_attr::tfloat:
            SQL << " float4";
            break;
         case tipo_attr::tdouble:
         case tipo_attr::tlongdouble:
            SQL << " float8";
            break;
         case tipo_attr::tshort:
         case tipo_attr::tunsignedshort:
            SQL << " int2";
            break;
         case tipo_attr::tlong:
         case tipo_attr::tunsignedlong:
            SQL << " int4";
            break;
         case tipo_attr::tlonglong:
         case tipo_attr::tunsignedlonglong:
            SQL << " int8";
            break;
         case tipo_attr::tchar:
         case tipo_attr::twchar:
            SQL << " char";
            break;
         case tipo_attr::tbool:
            SQL << " bool";
            break;
         case tipo_attr::tobject:
            cerr << "No se admite aun el tipo object" << endl;
            exit(-1);
            break;
         case tipo_attr::tstring:
         case tipo_attr::twstring:
            SQL << " text";
            break;
         case tipo_attr::tinterfaz:
            if (atr->readonly) {
               cerr << "No se admite como clave primaria una atributo que no sea de un tipo basico" << endl;
               exit(-1);
            }
            bool primeraclave=true;
            
            list<atributo *>::iterator it = atr->tipo.inter->atributos->begin();
            while ((*it)!=0) {
               if ((*it)->readonly) {
                  if (primeraclave)
                     primeraclave=false;
                  else
                     SQL << "," << endl << "\t" << atr->nombre;
                  
                  SQL << "_" << (*it)->nombre;
                  imprimeTipoSQL((*it),SQL);
                  SQL << " CONSTRAINT fk_" << atr->nombre << "_" << (*it)->nombre <<
                     " REFERENCES " << nombretabla(atr->tipo.inter->modulo,atr->tipo.inter->nombre,"_") 
                     << " (" << (*it)->nombre << ")";
               }
               it++;
            }
            break;
      }

}

void imprimeTipoIDL(atributo* atr, ofstream &IDL) {
      switch (atr->tipo.tipo) {
         case tipo_attr::tfloat:
            IDL << " float";
            break;
         case tipo_attr::tdouble:
            IDL << " double";
            break;
         case tipo_attr::tlongdouble:
            IDL << " long double";
            break;
         case tipo_attr::tshort:
            IDL << " short";
            break;
         case tipo_attr::tunsignedshort:
            IDL << " unsigned short";
            break;
         case tipo_attr::tlong:
            IDL << " long";
            break;
         case tipo_attr::tunsignedlong:
            IDL << " unsigned long";
            break;
         case tipo_attr::tlonglong:
            IDL << " long long";
            break;
         case tipo_attr::tunsignedlonglong:
            IDL << " unsigned long long";
            break;
         case tipo_attr::tchar:
            IDL << " char";
            break;
         case tipo_attr::twchar:
            IDL << " wchar";
            break;
         case tipo_attr::tbool:
            IDL << " boolean";
            break;
         case tipo_attr::tobject:
            cerr << "No se admite aun el tipo object" << endl;
            exit(-1);
            break;
         case tipo_attr::tstring:
            IDL << " string";
            break;
         case tipo_attr::twstring:
            IDL << " wstring";
            break;
      }
}

void generarIDL(interfaz *inter, ofstream &IDL) {
   cout << "Generando interfaces IDL para la clase " << inter->nombre << endl;
   int nmodulos = 0;
   int i;
      
   if (inter->modulo[0]!=0) {
      char *aux = new char[strlen(inter->modulo)+1];
      int j;
      for (i=0, j=0; i<=strlen(inter->modulo); i++, j++) {
         if (inter->modulo[i]=='.' || inter->modulo[i]==0) {
            aux[j]=0;
            IDL << "module " << aux << " {" << endl;
            j=-1;
            nmodulos++;
         }
         else
            aux[j]=inter->modulo[i];
      }
      delete [] aux;
   }
   
   // Se genera la interfaz nombreIterator
   IDL << "interface " << inter->nombre << "Iterator {" << endl;
   
   IDL << "\t" << nombretabla(inter->modulo,inter->nombre,"::") << " next();" << endl;
   
   IDL << "};" << endl;
   
   
   // Se genera la interfaz nombreFactory
   IDL << "interface " << inter->nombre << "Factory {" << endl;
   
   IDL << "\t" << nombretabla(inter->modulo,inter->nombre,"::") << " getObject(";
   
   list<atributo *>::iterator it = inter->atributos->begin();
   bool primeratr=true;
   while ((*it)!=0) {
      if ((*it)->readonly) {
         // Es un atributo primario
         if (primeratr)
            primeratr=false;
         else
            IDL << ", ";
         
         IDL << "in";
         imprimeTipoIDL((*it),IDL);
         IDL << " " << (*it)->nombre;
      }
      it++;
   }   
   
   IDL << ");" << endl;
   
   IDL << "\tvoid remove(";
   
   it = inter->atributos->begin();
   primeratr=true;
   while ((*it)!=0) {
      if ((*it)->readonly) {
         // Es un atributo primario
         if (primeratr)
            primeratr=false;
         else
            IDL << ", ";
         
         IDL << "in";
         imprimeTipoIDL((*it),IDL);
         IDL << " " << (*it)->nombre;
      }
      it++;
   }   
   
   IDL << ");" << endl;

   IDL << "\tboolean exists(";
   
   it = inter->atributos->begin();
   primeratr=true;
   while ((*it)!=0) {
      if ((*it)->readonly) {
         // Es un atributo primario
         if (primeratr)
            primeratr=false;
         else
            IDL << ", ";
         
         IDL << "in";
         imprimeTipoIDL((*it),IDL);
         IDL << " " << (*it)->nombre;
      }
      it++;
   }   
   
   IDL << ");" << endl;
   
   IDL << "\t" << inter->nombre << "Iterator list(in string condition);" << endl;   
   
   IDL << "};" << endl;

   for (i=0; i<nmodulos; i++) {
      IDL << "};" << endl;
   }
   
}

void generarImpl(interfaz *inter) {
   cout << "Generando Impl para la clase " << inter->nombre << endl;
   char *nombreClase = new char[strlen(inter->nombre) + strlen("Impl") + 1];
   
   strcpy(nombreClase,inter->nombre);
   strcat(nombreClase,"Impl");
   
   char *nombreFichero = new char[strlen(nombreClase) + strlen(".java") + 1];
   
   strcpy(nombreFichero,nombreClase);
   strcat(nombreFichero,".java");
   
   ofstream IMPL;
   IMPL.open(nombreFichero);
   
   delete [] nombreFichero;
   
   if (inter->modulo[0]!=0)
      IMPL << "package " << inter->modulo << ";" << endl;
      
   IMPL << "public class " << nombreClase << " extends _" << nombreClase << "Base" << endl;
   IMPL << "\timplements dbcollection.ChangeSensible {" << endl;
   
   list<atributo *>::iterator it = inter->atributos->begin();
   while ((*it)!=0) {
      IMPL << "\tprivate";
      imprimeTipoJava((*it),IMPL);
      IMPL << " " << (*it)->nombre << ";" << endl;
      it++;
   }   
   
   IMPL << "\tprivate boolean changed;" << endl;
   IMPL << "\tpublic boolean changed() { return changed; }" << endl;
   
   IMPL << "\tpublic " << nombreClase << "(";
   it = inter->atributos->begin();
   bool primeratr=true;
   while ((*it)!=0) {
      if (primeratr)
         primeratr=false;
      else
         IMPL << ",";

      imprimeTipoJava((*it),IMPL);
      IMPL << " " << (*it)->nombre;
      it++;
   }   
   
   IMPL << ") {" << endl;
   IMPL << "\t\tchanged = false;" << endl;
   
   it = inter->atributos->begin();
   while ((*it)!=0) {
      IMPL << "\t\tthis." << (*it)->nombre << " = " << (*it)->nombre << ";" << endl;
      it++;
   }   
   
   IMPL << "\t}" << endl << endl;

   it = inter->atributos->begin();
   while ((*it)!=0) {
      IMPL << "\tpublic";
      imprimeTipoJava((*it),IMPL);
      IMPL << " " << (*it)->nombre << "() { return " << (*it)->nombre << "; }" << endl;
      
      if (!(*it)->readonly) {
         IMPL << "\tpublic void " << (*it)->nombre << "(";
         imprimeTipoJava((*it),IMPL);
         IMPL << " valor) {" << endl;
         IMPL << "\t\tchanged = true;" << endl;
         IMPL << "\t\t" << (*it)->nombre << " = valor;" << endl;
         IMPL << "\t}" << endl;
      }
      
      IMPL << endl;
      
      it++;
   }   
   

   
   IMPL << "}";
   
   IMPL.close();
      
   delete [] nombreClase;
}


void imprimeTipoJava(atributo* atr, ofstream &SAL) {
      switch (atr->tipo.tipo) {
         case tipo_attr::tfloat:
            SAL << " float";
            break;
         case tipo_attr::tdouble:
         case tipo_attr::tlongdouble:
            SAL << " double";
            break;
         case tipo_attr::tshort:
         case tipo_attr::tunsignedshort:
            SAL << " short";
            break;
         case tipo_attr::tlong:
         case tipo_attr::tunsignedlong:
            SAL << " int";
            break;
         case tipo_attr::tlonglong:
         case tipo_attr::tunsignedlonglong:
            SAL << " long";
            break;
         case tipo_attr::tchar:
         case tipo_attr::twchar:
            SAL << " char";
            break;
         case tipo_attr::tbool:
            SAL << " boolean";
            break;
         case tipo_attr::tobject:
            cerr << "No se admite aun el tipo object" << endl;
            exit(-1);
            break;
         case tipo_attr::tstring:
         case tipo_attr::twstring:
            SAL << " String";
            break;
         case tipo_attr::tinterfaz:
            SAL << " " << atr->tipo.nombreinterfaz;
            break;
      }
}

void imprimeObjetoJava(atributo* atr, ofstream &SAL) {
      switch (atr->tipo.tipo) {
         case tipo_attr::tfloat:
            SAL << " new Float(" << atr->nombre << ")";
            break;
         case tipo_attr::tdouble:
         case tipo_attr::tlongdouble:
            SAL << " new Double(" << atr->nombre << ")";
            break;
         case tipo_attr::tshort:
         case tipo_attr::tunsignedshort:
            SAL << " new Short(" << atr->nombre << ")";
            break;
         case tipo_attr::tlong:
         case tipo_attr::tunsignedlong:
            SAL << " new Integer(" << atr->nombre << ")";
            break;
         case tipo_attr::tlonglong:
         case tipo_attr::tunsignedlonglong:
            SAL << " new Long(" << atr->nombre << ")";
            break;
         case tipo_attr::tchar:
         case tipo_attr::twchar:
            SAL << " new Character(" << atr->nombre << ")";
            break;
         case tipo_attr::tbool:
            SAL << " new Boolean(" << atr->nombre << ")";
            break;
         case tipo_attr::tobject:
            cerr << "No se admite aun el tipo object" << endl;
            exit(-1);
            break;
         case tipo_attr::tstring:
         case tipo_attr::twstring:
            SAL << " " << atr->nombre;
            break;
      }
}


void imprimeObjetoJava(tipo_attr tipo, ofstream &SAL) {
      switch (tipo.tipo) {
         case tipo_attr::tfloat:
            SAL << " new Float(" ;
            break;
         case tipo_attr::tdouble:
         case tipo_attr::tlongdouble:
            SAL << " new Double(" ;
            break;
         case tipo_attr::tshort:
         case tipo_attr::tunsignedshort:
            SAL << " new Short(" ;
            break;
         case tipo_attr::tlong:
         case tipo_attr::tunsignedlong:
            SAL << " new Integer(" ;
            break;
         case tipo_attr::tlonglong:
         case tipo_attr::tunsignedlonglong:
            SAL << " new Long(" ;
            break;
         case tipo_attr::tchar:
         case tipo_attr::twchar:
            SAL << " new Character(" ;
            break;
         case tipo_attr::tbool:
            SAL << " new Boolean(" ;
            break;
         case tipo_attr::tobject:
            cerr << "No se admite aun el tipo object" << endl;
            exit(-1);
            break;
         case tipo_attr::tstring:
         case tipo_attr::twstring:
            SAL << "( " ;
            break;
      }
}

void imprimeNuloSQL(atributo* atr, ofstream &SAL) {
      switch (atr->tipo.tipo) {
         case tipo_attr::tfloat:
         case tipo_attr::tdouble:
         case tipo_attr::tlongdouble:
            SAL << "0.0";
            break;
         case tipo_attr::tshort:
         case tipo_attr::tunsignedshort:
         case tipo_attr::tlong:
         case tipo_attr::tunsignedlong:
         case tipo_attr::tlonglong:
         case tipo_attr::tunsignedlonglong:
            SAL << "0";
            break;
         case tipo_attr::tbool:
            SAL << "false";
            break;
         case tipo_attr::tobject:
            cerr << "No se admite aun el tipo object" << endl;
            exit(-1);
            break;
         case tipo_attr::tchar:
         case tipo_attr::twchar:
         case tipo_attr::tstring:
         case tipo_attr::twstring:
            break;
         case tipo_attr::tinterfaz:
            bool primeraclave=true;
            
            list<atributo *>::iterator it = atr->tipo.inter->atributos->begin();
            while ((*it)!=0) {
               if ((*it)->readonly) {
                  if (primeraclave)
                     primeraclave=false;
                  else
                     SAL << "' , '";
                  
                  imprimeNuloSQL((*it),SAL);
               }
               it++;
            }
            break;

      }
}


void generarFactoryImpl(interfaz *inter) {
   cout << "Generando FactoryImpl para la clase " << inter->nombre << endl;
   
   char *nombreClase = new char[strlen(inter->nombre) + strlen("FactoryImpl") + 1];
   
   strcpy(nombreClase,inter->nombre);
   strcat(nombreClase,"FactoryImpl");
   
   char *nombreFichero = new char[strlen(nombreClase) + strlen(".java") + 1];
   
   strcpy(nombreFichero,nombreClase);
   strcat(nombreFichero,".java");
   
   ofstream IMPL;
   IMPL.open(nombreFichero);
   
   delete [] nombreFichero;
   
   if (inter->modulo[0]!=0)
      IMPL << "package " << inter->modulo << ";" << endl;
      
   IMPL << "import java.util.Vector;" << endl << endl;
   IMPL << "public class " << nombreClase << " extends _" << nombreClase << "Base {" << endl;
   IMPL << "\tprivate org.omg.CORBA.ORB orb;" << endl;
   IMPL << "\tprivate " << inter->nombre << "OA oa;" << endl;
   IMPL << "\tprivate dbcollection.AbstractFactory af;" << endl;
   
   IMPL << endl << "\tpublic " << nombreClase << "(org.omg.CORBA.ORB orb) {" << endl;
   IMPL << "\t\tsuper();" << endl;
   IMPL << "\t\tthis.orb = orb;" << endl;
   IMPL << "\t\taf = new dbcollection.AbstractFactory(orb, oa = new " << inter->nombre << "OA());" << endl;
   IMPL << "\t}" << endl << endl;
   
   IMPL << "\tpublic " << inter->nombre << " getObject(";
   
   imprimeClave(inter,IMPL);
   
   IMPL << ") {" << endl;
   
   IMPL << "\t\tVector key = new Vector();" << endl;
   
   imprimeGeneraClave(inter,IMPL);
   IMPL << "\t\treturn (" << inter->nombre;
   IMPL << ")(af.getObject(key));" << endl;
   
   IMPL << "\t}" << endl << endl;
   
   IMPL << "\tpublic void remove(";
   
   imprimeClave(inter,IMPL);
   
   IMPL << ") {" << endl;
   
   IMPL << "\t\tVector key = new Vector();" << endl;
   
   imprimeGeneraClave(inter,IMPL);
   IMPL << "\t\taf.remove(key);" << endl;
   
   IMPL << "\t}" << endl << endl;
   
   IMPL << "\tpublic boolean exists(";
   
   imprimeClave(inter,IMPL);
   
   IMPL << ") {" << endl;
   
   IMPL << "\t\tVector key = new Vector();" << endl;
   
   imprimeGeneraClave(inter,IMPL);
   IMPL << "\t\treturn af.exists(key);" << endl;
   
   IMPL << "\t}" << endl << endl;

   
   IMPL << "\tpublic " << inter->nombre << "Iterator list(String cond) {" << endl;
   
   IMPL << "\t\treturn new " << inter->nombre << "IteratorImpl(af,oa,cond,this,orb);" << endl;
   
   IMPL << "\t}" << endl << endl;
   
   IMPL << "\tpublic void finalize() {" << endl;
   IMPL << "\t\taf.finalize();" << endl;
   IMPL << "\t}" << endl; 

   
   IMPL << "}";
   
   IMPL.close();
      
   delete [] nombreClase;
   
}

void imprimeClave(interfaz *inter,ofstream &IMPL) {
   list<atributo *>::iterator it = inter->atributos->begin();
   bool primeratr=true;
   while ((*it)!=0) {
      if ((*it)->readonly) {
         if (primeratr)
            primeratr=false;
         else
            IMPL << ",";

         imprimeTipoJava((*it),IMPL);
         IMPL << " " << (*it)->nombre;
      }
      it++;
   }   
}


void imprimeGeneraClave(interfaz *inter,ofstream &IMPL) {
   list<atributo *>::iterator it = inter->atributos->begin();
   while ((*it)!=0) {
      if ((*it)->readonly) {
         IMPL << "\t\tkey.add(";
         imprimeObjetoJava((*it),IMPL);
         IMPL << ");" << endl;
      }
      it++;
   }
}


void generarOA(interfaz *inter) {
   int i;
   cout << "Generando OA para la clase " << inter->nombre << endl;
   
   char *nombreClase = new char[strlen(inter->nombre) + strlen("OA") + 1];
   
   strcpy(nombreClase,inter->nombre);
   strcat(nombreClase,"OA");
   
   char *nombreFichero = new char[strlen(nombreClase) + strlen(".java") + 1];
   
   strcpy(nombreFichero,nombreClase);
   strcat(nombreFichero,".java");
   
   ofstream IMPL;
   IMPL.open(nombreFichero);
   
   delete [] nombreFichero;
   
   if (inter->modulo[0]!=0)
      IMPL << "package " << inter->modulo << ";" << endl;
      
   IMPL << "import java.util.Vector;" << endl;
   IMPL << "import java.util.Enumeration;" << endl;
   IMPL << "import java.sql.*;" << endl << endl;
   IMPL << "public class " << nombreClase << " implements dbcollection.dbObjectAdaptor {" << endl;

   IMPL << "\tprivate Statement localSt;" << endl;
   IMPL << "\tprivate Statement pSt;" << endl;
   IMPL << "\tprivate ResultSet pRs;" << endl;
   IMPL << "\tprivate boolean hasMore = false;" << endl << endl;

   IMPL << "\tpublic " << nombreClase << "() {" << endl;
   IMPL << "\t\ttry {" << endl;
   IMPL << "\t\t\tlocalSt = dbcollection.servidorBD.baseDatos.createStatement();" << endl;
   IMPL << "\t\t\tpSt = dbcollection.servidorBD.baseDatos.createStatement();" << endl;
   IMPL << "\t\t} catch (SQLException ex) {" << endl;
   IMPL << "\t\t\tthrow new RuntimeException(\"Fallo en la base de datos\");" << endl;
   IMPL << "\t\t}" << endl;
   IMPL << "\t}" << endl << endl;
   
   IMPL << "\tpublic Object retrieve(Vector key) {" << endl;
   IMPL << "\t\ttry {" << endl;
   IMPL << "\t\t\tResultSet rs = localSt.executeQuery(\"SELECT * FROM ";
   IMPL << nombretabla(inter->modulo,inter->nombre,"_") << " WHERE ";
   imprimeGeneraSQL(inter,IMPL);
   IMPL << "\");" << endl;
   IMPL << "\t\t\tif (!rs.next()) {" << endl;
   
   list<atributo *>::iterator it = inter->atributos->begin();
   while ((*it)!=0) {
      if ((*it)->readonly) {
         IMPL << "\t\t\t\tif (";
         if ((*it)->tipo.tipo==tipo_attr::tstring || (*it)->tipo.tipo==tipo_attr::twstring) {
            IMPL << "!((String)key.get(0)).equals(\"\")) {" << endl;
         }
         else {
            imprimeCastingJava((*it),IMPL,0);
            IMPL << " != ";
            imprimeNuloJava((*it),IMPL);
            IMPL << ") {" << endl;
         }
         break;
      }
      it++;
   }

   IMPL << "\t\t\t\t\tlocalSt.executeUpdate(\"INSERT INTO ";
   IMPL << nombretabla(inter->modulo,inter->nombre,"_") << " VALUES (";
   imprimeGeneraValoresInsertSQL(inter,IMPL);
   IMPL << ")\");" << endl;
   IMPL << "\t\t\t\t\treturn new " << inter->nombre << "Impl(";
   imprimeGeneraValoresNuevoObjeto(inter,IMPL);
   IMPL << ");" << endl;
   IMPL << "\t\t\t\t} else" << endl;
   IMPL << "\t\t\t\t\treturn null;" << endl;
   IMPL << "\t\t\t} else {" << endl;
   i=1;
   it = inter->atributos->begin();
   bool primeratr=true;
   while ((*it)!=0) {
      if ((*it)->tipo.tipo==tipo_attr::tinterfaz) {
        
         if (primeratr) {
            primeratr=false;
            IMPL << "\t\t\t\tVector key2 = new Vector();" << endl;
         }
         else
            IMPL << "\t\t\t\tkey2 = new Vector();" << endl;

         int j=i;
         list<atributo *>::iterator it2 = (*it)->tipo.inter->atributos->begin();
         while ((*it2)!=0) {
            if ((*it2)->readonly) {
               IMPL << "\t\t\t\tkey2.add(";
               imprimeObjetoJava((*it2)->tipo, IMPL);
               imprimeObtenerDatos((*it2),IMPL,i,"rs");
               IMPL << "));" << endl;
               i++;
            }
            it2++;
         }
         IMPL << "\t\t\t\t" << (*it)->tipo.nombreinterfaz << " p" << j << " = (";
         IMPL << (*it)->tipo.nombreinterfaz <<")(new ";
         IMPL << (*it)->tipo.nombreinterfaz << "OA()).retrieve(key2);" << endl;
      }
      i++;
      it++;
   }
   
   
   IMPL << "\t\t\t\treturn new " << inter->nombre << "Impl(";
   i=1;
   it = inter->atributos->begin();
   primeratr=true;
   while ((*it)!=0) {
      if (primeratr)
         primeratr=false;
      else
         IMPL << ", ";

      if ((*it)->tipo.tipo==tipo_attr::tinterfaz) {
         IMPL << "p" << i;
         list<atributo *>::iterator it2 = (*it)->tipo.inter->atributos->begin();
         while ((*it2)!=0) {
            if ((*it2)->readonly)
               i++;
            it2++;
         }
         i--;
      }
      else
         imprimeObtenerDatos((*it),IMPL,i,"rs");
      
      i++;
      it++;
   }
   IMPL << ");" << endl;
   IMPL << "\t\t\t}" << endl;
   IMPL << "\t\t} catch (SQLException ex) {" << endl;
   IMPL << "\t\t\treturn null;" << endl;
   IMPL << "\t\t}" << endl;
   IMPL << "\t}" << endl << endl;

   IMPL << "\tpublic void store(Object o) {" << endl;
   // Si no hay atributos que no sean primarios no hay que actualizar nada
   it = inter->atributos->begin();
   while ((*it)!=0) {
      if (!(*it)->readonly) {
         break;
      }
      it++;
   }
  if ((*it)!=0) {
   IMPL << "\t\t" << inter->nombre << " obj = (" << inter->nombre << ") o;" << endl;
   IMPL << "\t\ttry {" << endl;
   IMPL << "\t\t\tlocalSt.executeUpdate(\"UPDATE ";
   IMPL << nombretabla(inter->modulo,inter->nombre,"_") << " SET ";
   it = inter->atributos->begin();
   primeratr=true;
   while ((*it)!=0) {
      if (!(*it)->readonly) {
         if ((*it)->tipo.tipo==tipo_attr::tinterfaz) {
            list<atributo *>::iterator it2 = (*it)->tipo.inter->atributos->begin();
            while ((*it2)!=0) {
               if ((*it2)->readonly) {
                  if (primeratr)
                     primeratr=false;
                  else
                     IMPL << ", ";
                  IMPL << (*it)->nombre << "_" << (*it2)->nombre << " = '\" + ((obj.";
                  IMPL << (*it)->nombre << "()==null) ? \"\" : ";
                  imprimeObjetoJava((*it2)->tipo,IMPL);
                  IMPL << "obj.";
                  IMPL << (*it)->nombre << "()." << (*it2)->nombre << "()).toString()) + \"'";
               }
               it2++;
            }
         }
         else {
            if (primeratr)
               primeratr=false;
            else
               IMPL << ", ";
            IMPL << (*it)->nombre << " = '\" + obj." << (*it)->nombre << "() + \"'";
         }
      }
      it++;
   }
   IMPL << " WHERE ";
   it = inter->atributos->begin();
   primeratr=true;
   while ((*it)!=0) {
      if ((*it)->readonly) {
         if (primeratr)
            primeratr=false;
         else
            IMPL << " AND ";
            
         IMPL << (*it)->nombre << " = '\" + obj." << (*it)->nombre << "() + \"'";
      }
      it++;
   }
   IMPL << "\");" << endl;
   IMPL << "\t\t} catch (SQLException ex) {" << endl;
   IMPL << "\t\t\treturn;" << endl;
   IMPL << "\t\t}" << endl;
  }
   IMPL << "\t}" << endl << endl;
   
   IMPL << "\tpublic void remove(Vector key) {" << endl;
   IMPL << "\t\ttry {" << endl;
   IMPL << "\t\t\tlocalSt.executeUpdate(\"DELETE FROM ";
   IMPL << nombretabla(inter->modulo,inter->nombre,"_") << " WHERE ";
   imprimeGeneraSQL(inter,IMPL);
   IMPL << "\");" << endl;
   IMPL << "\t\t} catch (SQLException ex) {" << endl;
   IMPL << "\t\t\treturn;" << endl;
   IMPL << "\t\t}" << endl;
   IMPL << "\t}" << endl << endl;
   
   IMPL << "\tpublic boolean exists(Vector key) {" << endl;
   IMPL << "\t\ttry {" << endl;
   IMPL << "\t\t\tResultSet rs = localSt.executeQuery(\"SELECT * FROM ";
   IMPL << nombretabla(inter->modulo,inter->nombre,"_") << " WHERE ";
   imprimeGeneraSQL(inter,IMPL);
   IMPL << "\");" << endl;
   IMPL << "\t\t\treturn rs.next();" << endl;
   IMPL << "\t\t} catch (SQLException ex) {" << endl;
   IMPL << "\t\t\treturn false;" << endl;
   IMPL << "\t\t}" << endl;
   IMPL << "\t}" << endl << endl;

   IMPL << "\tpublic boolean hasMoreElements() {" << endl;
   IMPL << "\t\treturn hasMore;" << endl;
   IMPL << "\t}" << endl << endl;
   
   IMPL << "\tpublic Enumeration list(String cond) {" << endl;
   IMPL << "\t\ttry {" << endl;
   IMPL << "\t\t\tString queryStr = \"SELECT * FROM ";
   IMPL << nombretabla(inter->modulo,inter->nombre,"_") << "\";" << endl;
   IMPL << "\t\t\tif (!cond.equals(\"\")) {" << endl;
   IMPL << "\t\t\t\tqueryStr = queryStr + \" WHERE \" + cond;" << endl;
   IMPL << "\t\t\t}" << endl;
   IMPL << "\t\t\tpRs = pSt.executeQuery(queryStr);" << endl;
   IMPL << "\t\t\thasMore = pRs.next();" << endl;
   IMPL << "\t\t\treturn (Enumeration) this;" << endl;
   IMPL << "\t\t} catch (SQLException ex) {" << endl;
   IMPL << "\t\t\tthrow new RuntimeException(\"Error en la Base de Datos\");" << endl;
   IMPL << "\t\t}" << endl;
   IMPL << "\t}" << endl << endl;
   
   IMPL << "\tpublic Object nextElement() {" << endl;
   IMPL << "\t\tif (!hasMore) { throw new java.util.NoSuchElementException(); }" << endl;
   IMPL << "\t\ttry {" << endl;
   i=1;
   it = inter->atributos->begin();
   primeratr=true;
   while ((*it)!=0) {
      if ((*it)->tipo.tipo==tipo_attr::tinterfaz) {
         if (primeratr) {
            primeratr=false;
            IMPL << "\t\t\tVector key2 = new Vector();" << endl;
         }
         else
            IMPL << "\t\t\tkey2 = new Vector();" << endl;
         int j=i;
         list<atributo *>::iterator it2 = (*it)->tipo.inter->atributos->begin();
         while ((*it2)!=0) {
            if ((*it2)->readonly) {
               IMPL << "\t\t\tkey2.add(";
               imprimeObjetoJava((*it2)->tipo, IMPL);
               imprimeObtenerDatos((*it2),IMPL,i,"pRs");
               IMPL << "));" << endl;
               i++;
            }
            it2++;
         }
         
         IMPL << "\t\t\t" << (*it)->tipo.nombreinterfaz << " p" << j << " = (";
         IMPL << (*it)->tipo.nombreinterfaz <<")(new ";
         IMPL << (*it)->tipo.nombreinterfaz << "OA()).retrieve(key2);" << endl;
      }
      i++;
      it++;
   }

   IMPL << "\t\t\tObject tmpObj = new " << inter->nombre << "Impl(";
   i=1;
   it = inter->atributos->begin();
   primeratr=true;
   while ((*it)!=0) {
      if (primeratr)
         primeratr=false;
      else
         IMPL << ", ";

      if ((*it)->tipo.tipo==tipo_attr::tinterfaz) {
         IMPL << "p" << i;
         list<atributo *>::iterator it2 = (*it)->tipo.inter->atributos->begin();
         while ((*it2)!=0) {
            if ((*it2)->readonly)
               i++;
            it2++;
         }
         i--;

      }
      else
         imprimeObtenerDatos((*it),IMPL,i,"pRs");
      
      i++;
      it++;
   }
   IMPL << ");" << endl;
   IMPL << "\t\t\thasMore = pRs.next();" << endl;
   IMPL << "\t\t\treturn tmpObj;" << endl;
   IMPL << "\t\t} catch (SQLException ex) {" << endl;
   IMPL << "\t\t\tthrow new RuntimeException(\"Error en la Base de Datos\");" << endl;
   IMPL << "\t\t}" << endl;
   IMPL << "\t}" << endl << endl;


   IMPL << "}";
   
   IMPL.close();
      
   delete [] nombreClase;
   
}

void imprimeGeneraSQL(interfaz *inter,ofstream &IMPL) {
   int nclave = 0;
   list<atributo *>::iterator it = inter->atributos->begin();
   bool primeratr=true;
   while ((*it)!=0) {
      if ((*it)->readonly) {
         if (primeratr)
            primeratr=false;
         else
            IMPL << " AND ";

         IMPL << (*it)->nombre << " = '\" + key.get(" << nclave << ").toString() + \"'";
         nclave++;
      }
      it++;
   }
}

void imprimeGeneraValoresInsertSQL(interfaz *inter,ofstream &IMPL) {
   int nclave = 0;
   list<atributo *>::iterator it = inter->atributos->begin();
   bool primeratr=true;
   while ((*it)!=0) {
      if (primeratr)
         primeratr=false;
      else
         IMPL << " , ";
      if ((*it)->readonly) {
         IMPL << "'\" + key.get(" << nclave << ").toString() + \"'";
         nclave++;
      }
      else {
         IMPL << "'";
         imprimeNuloSQL((*it),IMPL);
         IMPL << "'";
      }
      it++;
   }
}

void imprimeGeneraValoresNuevoObjeto(interfaz *inter,ofstream &IMPL) {
   int nclave = 0;
   list<atributo *>::iterator it = inter->atributos->begin();
   bool primeratr=true;
   while ((*it)!=0) {
      if (primeratr)
         primeratr=false;
      else
         IMPL << " , ";
      if ((*it)->readonly) {
         imprimeCastingJava((*it),IMPL,nclave);
         nclave++;
      }
      else {
         imprimeNuloJava((*it),IMPL);
      }
      it++;
   }
}


void imprimeNuloJava(atributo* atr, ofstream &SAL) {
      switch (atr->tipo.tipo) {
         case tipo_attr::tfloat:
            SAL << "(float)0.0";
            break;
         case tipo_attr::tdouble:
         case tipo_attr::tlongdouble:
            SAL << "0.0";
            break;
         case tipo_attr::tshort:
         case tipo_attr::tunsignedshort:
            SAL << "(short)0";
            break;
         case tipo_attr::tlong:
         case tipo_attr::tunsignedlong:
         case tipo_attr::tlonglong:
         case tipo_attr::tunsignedlonglong:
            SAL << "0";
            break;
         case tipo_attr::tchar:
         case tipo_attr::twchar:
            SAL << "(char)0";
            break;
         case tipo_attr::tbool:
            SAL << "false";
            break;
         case tipo_attr::tobject:
            cerr << "No se admite aun el tipo object" << endl;
            exit(-1);
            break;
         case tipo_attr::tstring:
         case tipo_attr::twstring:
            SAL << "\"\"";
            break;
         case tipo_attr::tinterfaz:
            SAL << "null";
            break;

      }
}


void imprimeCastingJava(atributo* atr, ofstream &SAL, int nclave) {
      switch (atr->tipo.tipo) {
         case tipo_attr::tfloat:
            SAL << "((Float)(key.get(" << nclave << "))).floatValue()";
            break;
         case tipo_attr::tdouble:
         case tipo_attr::tlongdouble:
            SAL << "((Double)(key.get(" << nclave << "))).doubleValue()";
            break;
         case tipo_attr::tshort:
         case tipo_attr::tunsignedshort:
            SAL << "((Short)(key.get(" << nclave << "))).shortValue()";
            break;
         case tipo_attr::tlong:
         case tipo_attr::tunsignedlong:
            SAL << "((Integer)(key.get(" << nclave << "))).intValue()";
            break;
         case tipo_attr::tlonglong:
         case tipo_attr::tunsignedlonglong:
            SAL << "((Long)(key.get(" << nclave << "))).longValue()";
            break;
         case tipo_attr::tchar:
         case tipo_attr::twchar:
            SAL << "((Character)(key.get(" << nclave << "))).charValue()";
            break;
         case tipo_attr::tbool:
            SAL << "((Boolean)(key.get(" << nclave << "))).booleanValue()";
            break;
         case tipo_attr::tobject:
            cerr << "No se admite aun el tipo object" << endl;
            exit(-1);
            break;
         case tipo_attr::tstring:
         case tipo_attr::twstring:
            SAL << "(String)(key.get(" << nclave << "))";
            break;
         case tipo_attr::tinterfaz:
            SAL << "(" << atr->tipo.nombreinterfaz << ")";
            break;
      }
}


void imprimeObtenerDatos(atributo* atr, ofstream &SAL, int i, char *resultset){
      switch (atr->tipo.tipo) {
         case tipo_attr::tfloat:
            SAL << resultset << ".getFloat(" << i << ")";
            break;
         case tipo_attr::tdouble:
         case tipo_attr::tlongdouble:
            SAL << resultset << ".getDouble(" << i << ")";
            break;
         case tipo_attr::tshort:
         case tipo_attr::tunsignedshort:
            SAL << resultset << ".getShort(" << i << ")";
            break;
         case tipo_attr::tlong:
         case tipo_attr::tunsignedlong:
            SAL << resultset << ".getInt(" << i << ")";
            break;
         case tipo_attr::tlonglong:
         case tipo_attr::tunsignedlonglong:
            SAL << resultset << ".getLong(" << i << ")";
            break;
         case tipo_attr::tchar:
         case tipo_attr::twchar:
            SAL << resultset << ".getString(" << i << ").charAt(0)";
            break;
         case tipo_attr::tbool:
            SAL << resultset << ".getBoolean(" << i << ")";
            break;
         case tipo_attr::tobject:
            cerr << "No se admite aun el tipo object" << endl;
            exit(-1);
            break;
         case tipo_attr::tstring:
         case tipo_attr::twstring:
            SAL << resultset << ".getString(" << i << ")";
            break;
         case tipo_attr::tinterfaz:
            SAL << "null";
            break;
      }
}

void generarIteratorImpl(interfaz *inter) {

   cout << "Generando IteratorImpl para la clase " << inter->nombre << endl;
   
   char *nombreClase = new char[strlen(inter->nombre) + strlen("IteratorImpl") + 1];
   
   strcpy(nombreClase,inter->nombre);
   strcat(nombreClase,"IteratorImpl");
   
   char *nombreFichero = new char[strlen(nombreClase) + strlen(".java") + 1];
   
   strcpy(nombreFichero,nombreClase);
   strcat(nombreFichero,".java");
   
   ofstream IMPL;
   IMPL.open(nombreFichero);
   
   delete [] nombreFichero;
   
   if (inter->modulo[0]!=0)
      IMPL << "package " << inter->modulo << ";" << endl;
      
   IMPL << "import dbcollection.AbstractFactory;" << endl;
   IMPL << "import java.util.Vector;" << endl;
   IMPL << "import java.util.Enumeration;" << endl << endl;
   IMPL << "public class " << nombreClase << " extends _" << nombreClase << "Base {" << endl;
   IMPL << "\tprivate String cond;" << endl;
   IMPL << "\tprivate AbstractFactory af;" << endl;
   IMPL << "\tprivate Enumeration e;" << endl;
   IMPL << "\tprivate " << inter->nombre << "Factory fac;" << endl;
   IMPL << "\tprivate org.omg.CORBA.ORB orb;" << endl << endl;
   
   IMPL << "\tpublic " << nombreClase << "(AbstractFactory af, " << inter->nombre << "OA oa, ";
   IMPL << "String cond, " << inter->nombre << "Factory fac, org.omg.CORBA.ORB orb) {" << endl;
   IMPL << "\t\te = oa.list(cond);" << endl;
   IMPL << "\t\tthis.af = af;" << endl;
   IMPL << "\t\tthis.cond = cond;" << endl;
   IMPL << "\t\tthis.fac = fac;" << endl;
   IMPL << "\t\tthis.orb = orb;" << endl;
   IMPL << "\t}" << endl << endl;
   
   IMPL << "\tpublic " << inter->nombre << " next() {" << endl;
   IMPL << "\t\tif (!e.hasMoreElements()) { orb.disconnect(this); return null; }" << endl;
   IMPL << "\t\tObject tmpObj = e.nextElement();" << endl;
   list<atributo *>::iterator it = inter->atributos->begin();
   while ((*it)!=0) {
      if ((*it)->readonly) {
         IMPL << "\t\t";
         imprimeTipoJava((*it),IMPL);
         IMPL << " " << (*it)->nombre << " = ((" << inter->nombre << ")tmpObj)." << (*it)->nombre;
         IMPL << "();" << endl;;
      }
      it++;
   }
   IMPL << "\t\tVector key = new Vector();" << endl;
   imprimeGeneraClave(inter,IMPL);
   IMPL << "\t\treturn (" << inter->nombre;
   IMPL << ")(af.cacheObject(key,tmpObj));" << endl;
   IMPL << "\t}" << endl;

   IMPL << "}";
   
   IMPL.close();
      
   delete [] nombreClase;
   
}

void generarSafe(interfaz *inter) {
   cout << "Generando Safe para la clase " << inter->nombre << endl;
   
   char *nombreClase = new char[strlen(inter->nombre) + strlen("Safe") + 1];
   
   strcpy(nombreClase,"Safe");
   strcat(nombreClase,inter->nombre);
   
   char *nombreFichero = new char[strlen(nombreClase) + strlen(".java") + 1];
   
   strcpy(nombreFichero,nombreClase);
   strcat(nombreFichero,".java");
   
   ofstream IMPL;
   IMPL.open(nombreFichero);
   
   delete [] nombreFichero;
   
   if (inter->modulo[0]!=0)
      IMPL << "package " << inter->modulo << ";" << endl;
      

   IMPL << "public class " << nombreClase <<" {" << endl;
   IMPL << "\tpublic " << inter->nombre << " ref;" << endl;
   IMPL << "\tprivate " << inter->nombre << "Factory factory;" << endl;
   list<atributo *>::iterator it = inter->atributos->begin();
   while ((*it)!=0) {
      if ((*it)->readonly) {
         IMPL << "\tprivate";
         imprimeTipoJava((*it),IMPL);
         IMPL << " " << (*it)->nombre << ";" << endl;
      }
      it++;
   }
   IMPL << endl;
   
   IMPL << "\tpublic " << nombreClase << "(" << inter->nombre << "Factory factory,";
   it = inter->atributos->begin();
   bool primeratr=true;
   while ((*it)!=0) {
      if (primeratr)
         primeratr=false;
      else
         IMPL << ",";

      imprimeTipoJava((*it),IMPL);
      IMPL << " " << (*it)->nombre;
      it++;
   }   
   IMPL << ") {" << endl;
   IMPL << "\t\tthis.factory = factory;" << endl;
   it = inter->atributos->begin();
   while ((*it)!=0) {
      if ((*it)->readonly) {
         IMPL << "\t\tthis." << (*it)->nombre << " = " << (*it)->nombre << ";" << endl;
      }
      it++;
   }
   IMPL << "\t\tref = factory.getObject(";
   it = inter->atributos->begin();
   primeratr=true;
   while ((*it)!=0) {
      if ((*it)->readonly) {
         if (primeratr)
            primeratr=false;
         else
            IMPL << ", ";

         IMPL << (*it)->nombre;
      }
      it++;
   }
   IMPL << ");" << endl;
   it = inter->atributos->begin();
   while ((*it)!=0) {
      if (!(*it)->readonly) {
         IMPL << "\t\t" << (*it)->nombre << "(" << (*it)->nombre << ");" << endl;
      }
      it++;
   }
   IMPL << "\t}" << endl << endl;
   
   // Si no hay atributos que no sean primarios no hay que generar el segundo constructor
   it = inter->atributos->begin();
   while ((*it)!=0) {
      if (!(*it)->readonly) {
         break;
      }
      it++;
   }
  if ((*it)!=0) {
   IMPL << "\tpublic " << nombreClase << "(" << inter->nombre << "Factory factory,";
   it = inter->atributos->begin();
   bool primeratr=true;
   while ((*it)!=0) {
      if ((*it)->readonly) {
         if (primeratr)
            primeratr=false;
         else
            IMPL << ",";

         imprimeTipoJava((*it),IMPL);
         IMPL << " " << (*it)->nombre;
      }
      it++;
   }   
   IMPL << ") {" << endl;
   IMPL << "\t\tthis.factory = factory;" << endl;
   it = inter->atributos->begin();
   while ((*it)!=0) {
      if ((*it)->readonly) {
         IMPL << "\t\tthis." << (*it)->nombre << " = " << (*it)->nombre << ";" << endl;
      }
      it++;
   }
   IMPL << "\t\tref = factory.getObject(";
   it = inter->atributos->begin();
   primeratr=true;
   while ((*it)!=0) {
      if ((*it)->readonly) {
         if (primeratr)
            primeratr=false;
         else
            IMPL << ", ";

         IMPL << (*it)->nombre;
      }
      it++;
   }
   IMPL << ");" << endl;
   IMPL << "\t}" << endl << endl;
  }
  
   IMPL << "\tpublic " << nombreClase << "(" << inter->nombre << "Factory factory, ";
   IMPL << inter->nombre << " ref) {" << endl;
   IMPL << "\t\tthis.factory = factory;" << endl;
   IMPL << "\t\tthis.ref = ref;" << endl;
   it = inter->atributos->begin();
   while ((*it)!=0) {
      if ((*it)->readonly) {
         IMPL << "\t\tthis." << (*it)->nombre << " = " << (*it)->nombre << "();" << endl;
      }
      it++;
   }
   IMPL << "\t}" << endl << endl;
   
   it = inter->atributos->begin();
   while ((*it)!=0) {
      IMPL << "\tpublic";
      imprimeTipoJava((*it),IMPL);
      IMPL << " " << (*it)->nombre << "() {" << endl;
      IMPL << "\t\twhile (true) {" << endl;
      IMPL << "\t\t\ttry {" << endl;
      IMPL << "\t\t\t\treturn ref." << (*it)->nombre << "();" << endl;
      IMPL << "\t\t\t} catch (org.omg.CORBA.OBJECT_NOT_EXIST ex) {" << endl;
      IMPL << "\t\t\t\tref = factory.getObject(";
      list<atributo *>::iterator it2 = inter->atributos->begin();
      primeratr=true;
      while ((*it2)!=0) {
         if ((*it2)->readonly) {
            if (primeratr)
               primeratr=false;
            else
               IMPL << ", ";
         
            IMPL << (*it2)->nombre;
         }
         it2++;
      }
      IMPL << ");" << endl;
      IMPL << "\t\t\t}" << endl;
      IMPL << "\t\t}" << endl;
      IMPL << "\t}" << endl << endl;
      
      if (!(*it)->readonly) {
         IMPL << "\tpublic void " << (*it)->nombre << "(";
         imprimeTipoJava((*it),IMPL);
         IMPL << " value) {" << endl;
         IMPL << "\t\twhile (true) {" << endl;
         IMPL << "\t\t\ttry {" << endl;
         IMPL << "\t\t\t\tref." << (*it)->nombre << "(value);" << endl;
         IMPL << "\t\t\t\treturn;" << endl;
         IMPL << "\t\t\t} catch (org.omg.CORBA.OBJECT_NOT_EXIST ex) {" << endl;
         IMPL << "\t\t\t\tref = factory.getObject(";
         it2 = inter->atributos->begin();
         primeratr=true;
         while ((*it2)!=0) {
            if ((*it2)->readonly) {
               if (primeratr)
                  primeratr=false;
               else
                  IMPL << ", ";
         
               IMPL << (*it2)->nombre;
            }
            it2++;
         }
         IMPL << ");" << endl;
         IMPL << "\t\t\t\t" << (*it)->nombre << "(value);" << endl;
         IMPL << "\t\t\t}" << endl;
         IMPL << "\t\t}" << endl;
         IMPL << "\t}" << endl << endl;
      }
      it++;
   }



   IMPL << "}";
   
   IMPL.close();
      
   delete [] nombreClase;


}


void generarServidor(list<interfaz*> *linter) {
   int i;
   
   cout << "Generando Servidor" << endl;
   
   char *nombreClase = new char[strlen("Servidor") + 1];
   
   strcpy(nombreClase,"Servidor");
   
   char *nombreFichero = new char[strlen(nombreClase) + strlen(".java") + 1];
   
   strcpy(nombreFichero,nombreClase);
   strcat(nombreFichero,".java");
   
   ofstream IMPL;
   IMPL.open(nombreFichero);
   
   delete [] nombreFichero;

   list<interfaz *>::iterator it = linter->begin();
   interfaz *inter1 = *it;
   
   if (inter1->modulo[0]!=0)
      IMPL << "package " << inter1->modulo << ";" << endl;
      
   IMPL << "import org.omg.CORBA.*;" << endl;
   IMPL << "import java.io.*;" << endl;
   IMPL << "import org.omg.CosNaming.*;" << endl << endl;
   
   IMPL << "public class Servidor {" << endl;
   IMPL << "\tpublic static void main(String args[]) {" << endl;
   IMPL << "\t\tORB orb = ORB.init(args,null);" << endl;
   IMPL << "\t\tNamingContext rootContext=null;" << endl;
   IMPL << "\t\ttry {" << endl;
   IMPL << "\t\t\torg.omg.CORBA.Object objRef = orb.resolve_initial_references(\"NameService\");" << endl;
   IMPL << "\t\t\trootContext = NamingContextHelper.narrow(objRef);" << endl;
   IMPL << "\t\t} catch (org.omg.CORBA.ORBPackage.InvalidName ex) {" << endl;
   IMPL << "\t\t\tSystem.err.println(\"Error, no se encuentra el servicio de nombres\");" << endl;
   IMPL << "\t\t\tSystem.exit(-1);" << endl;
   IMPL << "\t\t}" << endl;
   IMPL << "\t\ttry {" << endl;
   IMPL << "\t\t\tNameComponent modContext[] = new NameComponent[1];" << endl;
   IMPL << "\t\t\tNamingContext servidores = rootContext;" << endl;
   int principiomod=0;
   char *copiamod = new char[strlen(inter1->modulo)];
   strcpy (copiamod,inter1->modulo);
   for (i=0; i<=strlen(inter1->modulo); i++) {
      if (inter1->modulo[i]=='.' || inter1->modulo[i]==0) {
         copiamod[i]=0;
         
         IMPL << "\t\t\tmodContext[0] = new NameComponent(\"" << &copiamod[principiomod] << "\", \"\");" << endl;
         IMPL << "\t\t\ttry {" << endl;
         IMPL << "\t\t\t\tservidores = NamingContextHelper.narrow(servidores.resolve(modContext));" << endl;
         IMPL << "\t\t\t} catch (org.omg.CosNaming.NamingContextPackage.NotFound ex) {" << endl;
         IMPL << "\t\t\t\tservidores = servidores.bind_new_context(modContext);" << endl;
         IMPL << "\t\t\t}" << endl;
         
         principiomod=i+1;
      }
   }
   
   while ((*it)!=0) {
      IMPL << "\t\t\t" << (*it)->nombre << "FactoryImpl fact" << (*it)->nombre << " = new ";
      IMPL << (*it)->nombre << "FactoryImpl(orb);" << endl;
      IMPL << "\t\t\torb.connect(fact" << (*it)->nombre << ");" << endl;
      IMPL << "\t\t\tmodContext[0] = new NameComponent(\"" << (*it)->nombre;
      IMPL << "FactoryImpl\", \"\");" << endl;
      IMPL << "\t\t\tservidores.rebind(modContext, fact" << (*it)->nombre << ");" << endl;
      it++;
   }

   IMPL << "\t\t\tSystem.out.println(\"Servidor  a la escucha\");" << endl;
   IMPL << "\t\t\tSystem.out.println(\"Pulse ENTER para finalizar\");" << endl;
   IMPL << "\t\t\ttry {" << endl;
   IMPL << "\t\t\t\tBufferedReader teclado;" << endl;
   IMPL << "\t\t\t\tteclado = new BufferedReader(new InputStreamReader(System.in));" << endl;
   IMPL << "\t\t\t\tString linea=teclado.readLine();" << endl;
   IMPL << "\t\t\t} catch (IOException ex) {}" << endl;
   it = linter->begin();
   while ((*it)!=0) {
      IMPL << "\t\t\tfact" << (*it)->nombre << ".finalize();" << endl;
      it++;
   }
   IMPL << "\t\t} catch (org.omg.CORBA.UserException ex) {" << endl;
   IMPL << "\t\t\tSystem.err.println(\"Excepcion no esperada, finalizacion anormal\");" << endl;
   IMPL << "\t\t\tex.printStackTrace();" << endl;
   IMPL << "\t\t}" << endl;
   IMPL << "\t}" << endl;

   IMPL << "}";
   
   IMPL.close();
      
   delete [] nombreClase;   

}
